create_local_socket(): Avoid code duplication.
[paraslash.git] / net.c
diff --git a/net.c b/net.c
index 8a190ddaa05100190761a7eecd918d78a546bb4d..9698e427868e212646f682d9b8dc2d7e8f6235ae 100644 (file)
--- a/net.c
+++ b/net.c
@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2014 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2005 Andre Noll <maan@tuebingen.mpg.de>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
@@ -48,7 +48,7 @@
  * \param netmask Value of the netmask part in \a cidr or the
  *               default of 32 if not specified.
  *
- * \return Pointer to \a addr if succesful, NULL on error.
+ * \return Pointer to \a addr if successful, NULL on error.
  * \sa RFC 4632
  */
 char *parse_cidr(const char *cidr,
@@ -208,7 +208,7 @@ const char *stringify_port(int port, const char *transport)
                struct servent *se = getservbyport(htons(port), transport);
 
                if (se == NULL)
-                       snprintf(service, sizeof(service), "%u", port);
+                       snprintf(service, sizeof(service), "%d", port);
                else
                        snprintf(service, sizeof(service), "%s", se->s_name);
        }
@@ -601,7 +601,7 @@ static inline int estimated_header_overhead(const int af_type)
  *
  * \param sockfd The socket file descriptor.
  *
- * The socket must be connected. See RFC 1122, 3.3.3. If the protocol familiy
+ * The socket must be connected. See RFC 1122, 3.3.3. If the protocol family
  * could not be determined, \p AF_INET is assumed.
  *
  * \return The maximum message size of the address family type.
@@ -622,11 +622,9 @@ int generic_max_transport_msg_size(int sockfd)
 }
 
 /**
- * Look up the local or remote side of a connected socket structure.
+ * Look up the remote side of a connected socket structure.
  *
  * \param fd The socket descriptor of the connected socket.
- * \param getname Either \p getsockname() for local, or \p getpeername() for
- * remote side.
  *
  * \return A static character string identifying hostname and port of the
  * chosen side in numeric host:port format.
@@ -634,7 +632,7 @@ int generic_max_transport_msg_size(int sockfd)
  * \sa getsockname(2), getpeername(2), parse_url(), getnameinfo(3),
  * services(5), nsswitch.conf(5).
  */
-static char *__get_sock_name(int fd, typeof(getsockname) getname)
+char *remote_name(int fd)
 {
        struct sockaddr_storage ss;
        const struct sockaddr *sa;
@@ -643,7 +641,7 @@ static char *__get_sock_name(int fd, typeof(getsockname) getname)
        static char output[sizeof(hbuf) + sizeof(sbuf) + 4];
        int ret;
 
-       if (getname(fd, (struct sockaddr *)&ss, &sslen) < 0) {
+       if (getpeername(fd, (struct sockaddr *)&ss, &sslen) < 0) {
                PARA_ERROR_LOG("can not determine address from fd %d: %s\n",
                        fd, strerror(errno));
                snprintf(output, sizeof(output), "(unknown)");
@@ -663,36 +661,6 @@ static char *__get_sock_name(int fd, typeof(getsockname) getname)
        return output;
 }
 
-/**
- * Look up the local side of a connected socket structure.
- *
- * \param sockfd The file descriptor of the socket.
- *
- * \return A pointer to a static buffer containing hostname an port. This
- * buffer must not be freed by the caller.
- *
- * \sa remote_name().
- */
-char *local_name(int sockfd)
-{
-       return __get_sock_name(sockfd, getsockname);
-}
-
-/**
- * Look up the remote side of a connected socket structure.
- *
- * \param sockfd The file descriptor of the socket.
- *
- * \return Analogous to the return value of \ref local_name() but for the
- * remote side.
- *
- * \sa local_name().
- */
-char *remote_name(int sockfd)
-{
-       return __get_sock_name(sockfd, getpeername);
-}
-
 /**
  * Extract IPv4 or IPv6-mapped-IPv4 address from sockaddr_storage.
  *
@@ -877,34 +845,36 @@ static int init_unix_addr(struct sockaddr_un *u, const char *name)
 }
 
 /**
* Prepare, create, and bind a socket for local communication.
  *
  * \param name The socket pathname.
- * \param unix_addr Pointer to the \p AF_UNIX socket structure.
- * \param mode The desired mode of the socket.
+ * \param mode The desired permissions of the socket.
  *
* This function creates a local socket for sequenced, reliable,
* two-way, connection-based byte streams.
  *
* \return The file descriptor, on success, negative on errors.
  *
- * \sa socket(2)
- * \sa bind(2)
- * \sa chmod(2)
+ * \sa socket(2), \sa bind(2), \sa chmod(2), listen(2), unix(7).
  */
-int create_local_socket(const char *name, struct sockaddr_un *unix_addr,
-               mode_t mode)
+int create_local_socket(const char *name, mode_t mode)
 {
+       struct sockaddr_un unix_addr;
        int fd, ret;
 
-       ret = init_unix_addr(unix_addr, name);
+       ret = init_unix_addr(&unix_addr, name);
        if (ret < 0)
                return ret;
        ret = socket(PF_UNIX, SOCK_STREAM, 0);
        if (ret < 0)
                return -ERRNO_TO_PARA_ERROR(errno);
        fd = ret;
-       ret = bind(fd, (struct sockaddr *) unix_addr, UNIX_PATH_MAX);
+       ret = mark_fd_nonblocking(fd);
+       if (ret < 0)
+               goto err;
+       ret = bind(fd, (struct sockaddr *)&unix_addr, UNIX_PATH_MAX);
        if (ret < 0) {
                ret = -ERRNO_TO_PARA_ERROR(errno);
                goto err;
@@ -912,6 +882,10 @@ int create_local_socket(const char *name, struct sockaddr_un *unix_addr,
        ret = -E_CHMOD;
        if (chmod(name, mode) < 0)
                goto err;
+       if (listen(fd , 5) < 0) {
+               ret = -ERRNO_TO_PARA_ERROR(errno);
+               goto err;
+       }
        return fd;
 err:
        close(fd);
@@ -877,34 +845,36 @@ static int init_unix_addr(struct sockaddr_un *u, const char *name)
 
 /**
- * Prepare, create, and bind a socket for local communication.
+ * Create a socket for local communication and listen on it.
  *
  * \param name The socket pathname.
@@ -877,34 +845,36 @@ static int init_unix_addr(struct sockaddr_un *u, const char *name)
  *
- * This function creates a local socket for sequenced, reliable,
- * two-way, connection-based byte streams.
+ * This function creates a passive local socket for sequenced, reliable,
+ * two-way, connection-based byte streams. The socket file descriptor is set to
+ * nonblocking mode and listen(2) is called to prepare the socket for
+ * accepting incoming connection requests.
  *
@@ -877,34 +845,36 @@ static int init_unix_addr(struct sockaddr_un *u, const char *name)
  *
- * \return The file descriptor, on success, negative on errors.
+ * \return The file descriptor on success, negative error code on failure.
  *